import numpy as np
from scipy.ndimage import rotate
import imageio
from matplotlib import pyplot as plt


# Define projection operation

def proj(a, theta):
    """
    Project tomogram slice a along angle theta (in degrees)
    """
    return rotate(a, theta, reshape=False).sum(axis=0)


def radon(a, theta_list):
    """
    Compute Radon transform of a for given angle list (degrees)
    """
    sino = np.empty((len(theta_list), a.shape[0]))

    for i, theta in enumerate(theta_list):
        sino[i, :] = proj(a, theta)

    return sino


def backproj(p, theta):
    """
    Back-project 1D signal p at angle theta.
    """
    N = len(p)
    bp0 = np.tile(p, (N, 1))
    bp = rotate(bp0, -theta, reshape=False)
    x, y = np.ogrid[-1:1:1j*N, -1:1:1j*N]
    mask = ((x**2 + y**2) <= 1)
    return bp * mask


def FBP(sino, theta_list):
    """
    Compute inverse Radon transform using the Filtered back-projection method.
    """
    N = sino.shape[1]

    # FFT of sinogram
    sino_f = np.fft.fft(sino, axis=1)

    # Compute filter
    filter = np.abs(np.fft.fftfreq(N))

    # Apply filter and inverse FFT
    sino_filtered = np.real(np.fft.ifft(sino_f*filter, axis=1))

    # Back-project
    tomo = np.zeros((N, N))

    for i, line in enumerate(sino_filtered):
        tomo += backproj(line, theta_list[i])

    return tomo
